//faultTolerantCN.java

import java.io.*;
import java.lang.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

class Globals
{
	static int MobileHostID=0;
	static int MyChannelID=0;
	static int MSSWidth=10;
	static int MSSHeight=10;
	static int HexWidth=70; //cell-hexagon width
	static int HexHeight=90; //cell-hexagon height
	static int CellStartX=95; //firstcell's xpos
	static int CellStartY=140; //firstcell's ypos
}

class ChannelStatus
{	
	static int Allocated=0;
	static int Busy=1;
	
	static String[] StatusString;
	static
	{
		StatusString=new String[]
		{
			new String("Allocated"),
			new String("Busy"),
		};
	}
}

class MobileHost
{
	int id;
	int x;
	int y;
	
	MobileHost(int tx,int ty)
	{
		id=Globals.MobileHostID;
		x=tx;
		y=ty;
		Globals.MobileHostID++;
	}
	
	public int getid()
	{
		return(id);
	}
	
	public int getx()
	{
		return(x);
	}

	public int gety()
	{
		return(y);
	}
}

class MyChannel
{
	int id;
	int x;
	int y;
	int status;
	
	MyChannel(int tx,int ty,int tstatus)
	{
		id=Globals.MyChannelID;
		Globals.MyChannelID++;
		x=tx;
		y=ty;
		status=tstatus;
	}
	
	public int getx()
	{
		return(x);
	}

	public int gety()
	{
		return(y);
	}
	
	public int getstatus()
	{
		return(status);
	}
}

class Cell
{
	int id;		//unique id for each cell
	int x,y;	//display position
	int AdjCount;
	int AdjCells[]=new int[6]; //maximum 6 adjacent cells for a cell
	MobileHost[] MobileHosts=new MobileHost[100];
	int MobileHostCount;
	MyChannel[] MyChannels=new MyChannel[100];
	int MyChannelCount;
	int LinkedMH[]=new int[100];
	int GroupCells[]=new int[100];
	int GroupCellCount;

	//constructor
	Cell()
	{
		id=-1;
		AdjCount=0;
		MobileHostCount=0;
		MyChannelCount=0;
		GroupCellCount=0;
	}
	
	//set functions
	public void setid(int tid)
	{
		id=tid;
	}
	public void setxy(int tx,int ty)
	{
		x=tx;
		y=ty;
	}
	
	//get functions
	public int getid()
	{
		return(id);
	}

	public int getx()
	{
		return(x);
	}
	
	public int gety()
	{
		return(y);
	}
	
	public int getMobileHostCount() //MobileHosts get functions
	{
		return(MobileHostCount);
	}
	
	public MobileHost getMobileHost(int index)
	{
		return(MobileHosts[index]);
	}
	
	public int getMobileHostID(int index)
	{
		return(MobileHosts[index].id);
	}
	
	public int getMobileHostX(int index)
	{
		return(MobileHosts[index].x);
	}
	
	public int getMobileHostY(int index)
	{
		return(MobileHosts[index].y);
	}

	public int getMyChannelCount() //MyChannels get functions
	{
		return(MyChannelCount);
	}
	
	public MyChannel getMyChannel(int index)
	{
		return(MyChannels[index]);
	}
	
	public int getMyChannelLink(int index)
	{
		return(LinkedMH[index]);
	}
	
	public int getMyChannelID(int index)
	{
		return(MyChannels[index].id);
	}
	
	public int getMyChannelX(int index)
	{
		return(MyChannels[index].x);
	}
	
	public int getMyChannelY(int index)
	{
		return(MyChannels[index].y);
	}

	public int getMyChannelStatus(int index)
	{
		return(MyChannels[index].status);
	}
	
	public void setMyChannelStatus(int tIndex,int tStatus)
	{
		MyChannels[tIndex].status=tStatus;
	}

	public int getAdjCount() //AdjCell get functions
	{
		return(AdjCount);
	}

	public int getAdjCell(int tindex)
	{
		int tcellid=-1;
		if(tindex>=0&&tindex<=AdjCount-1) tcellid=AdjCells[tindex];
		return(tcellid);
	}

	public int getGroupCell(int tindex)
	{
		int tcellid=-1;
		if(tindex>=0&&tindex<=GroupCellCount-1) tcellid=GroupCells[tindex];
		return(tcellid);
	}

	//methods
	public void addAdjCell(int tcellid)
	{
		AdjCells[AdjCount]=tcellid;
		AdjCount++;
	}
	
	public void addMobileHost(int xposition,int yposition)
	{
	  	MobileHosts[MobileHostCount]=new MobileHost(xposition,yposition);
	  	LinkedMH[MobileHostCount]=-1;
	  	MobileHostCount++;
	}
	
	public boolean addMyChannel(int tx,int ty,int tstatus)
	{
	  	boolean flag=false;
	  	
	  	if(MobileHostCount!=0)
	  	{
		  	if(MyChannelCount<MobileHostCount)
		  	{
			  	MyChannels[MyChannelCount]=new MyChannel(tx,ty,tstatus);
		  		LinkedMH[MyChannelCount]=MobileHosts[MyChannelCount].getid();
			  	MyChannelCount++;
			  	flag=true;
			}
		}
		
		//return(flag);
		return(true);
	}
	
	public void addGroupCell(int tCellID)
	{
		GroupCells[GroupCellCount]=tCellID;
		GroupCellCount++;
	}
	
	public void draw(Graphics g)
	{
		//draw cell-hexagon
		g.setColor(new Color(0,0,255));
		Graphics2D g2d=(Graphics2D)g;
		g2d.setStroke(new BasicStroke(2.0f));
		g.drawLine(x,y,x+Globals.HexWidth,y);
		g.drawLine(x+Globals.HexWidth,y,x+Globals.HexWidth+Globals.HexWidth/2,y+Globals.HexHeight/2);
		g.drawLine(x,y,x-Globals.HexWidth/2,y+Globals.HexHeight/2);
		g.drawLine(x-Globals.HexWidth/2,y+Globals.HexHeight/2,x,y+Globals.HexHeight);
		g.drawLine(x+Globals.HexWidth+Globals.HexWidth/2,y+Globals.HexHeight/2,x+Globals.HexWidth,y+Globals.HexHeight);
		g.drawLine(x,y+Globals.HexHeight,x+Globals.HexWidth,y+Globals.HexHeight);
		
		//draw MSS
		int tCenterX=(Globals.HexWidth-Globals.MSSWidth)/2;
		int tCenterY=(Globals.HexHeight-Globals.MSSHeight)/2;
		g.drawRect(x+tCenterX,y+tCenterY,Globals.MSSWidth,Globals.MSSHeight);
		
		try
		{
			//Thread.sleep(1);
		}
		catch(Exception e)
		{
			System.out.println("Error: "+e.getMessage());
		}
	}
};

class Map
{
	int CellCount=7;	//number of cells
	Cell MapCells[];	//cells list

	//constructor
	Map()
	{
		MapCells=new Cell[CellCount];
		
		//set cell ids
		for(int t=0;t<CellCount;t++)
		{
			MapCells[t]=new Cell();
			MapCells[t].setid(t);
		}
	
		//set adjcent cells
		MapCells[0].addAdjCell(1);	//
		MapCells[0].addAdjCell(4);
		MapCells[0].addAdjCell(3);
		MapCells[1].addAdjCell(0);	//
		MapCells[1].addAdjCell(4);
		MapCells[1].addAdjCell(2);
		MapCells[2].addAdjCell(1);	//
		MapCells[2].addAdjCell(4);
		MapCells[2].addAdjCell(5);
		MapCells[3].addAdjCell(0);	//
		MapCells[3].addAdjCell(4);
		MapCells[3].addAdjCell(6);
		MapCells[4].addAdjCell(0);	//
		MapCells[4].addAdjCell(1);
		MapCells[4].addAdjCell(2);
		MapCells[4].addAdjCell(3);
		MapCells[4].addAdjCell(6);
		MapCells[4].addAdjCell(5);
		MapCells[5].addAdjCell(2);	//
		MapCells[5].addAdjCell(4);
		MapCells[5].addAdjCell(6);
		MapCells[6].addAdjCell(3);	//
		MapCells[6].addAdjCell(4);
		MapCells[6].addAdjCell(5);
			
		//set cells screen pos
		MapCells[0].setxy(Globals.CellStartX,Globals.CellStartY);
		MapCells[1].setxy(Globals.CellStartX+Globals.HexWidth+Globals.HexWidth/2,Globals.CellStartY-Globals.HexHeight/2);
		MapCells[2].setxy(Globals.CellStartX+Globals.HexWidth*3,Globals.CellStartY);
		MapCells[3].setxy(Globals.CellStartX,Globals.CellStartY+Globals.HexHeight);
		MapCells[4].setxy(Globals.CellStartX+Globals.HexWidth+Globals.HexWidth/2,Globals.CellStartY+Globals.HexHeight/2);
		MapCells[5].setxy(Globals.CellStartX+Globals.HexWidth*3,Globals.CellStartY+Globals.HexHeight);
		MapCells[6].setxy(Globals.CellStartX+Globals.HexWidth+Globals.HexWidth/2,Globals.CellStartY+Globals.HexHeight+Globals.HexHeight/2);
		
		//set cell groups
		MapCells[0].addGroupCell(2); //
		MapCells[0].addGroupCell(5);
		MapCells[0].addGroupCell(6);
		MapCells[1].addGroupCell(3); //
		MapCells[1].addGroupCell(5);
		MapCells[1].addGroupCell(6);
		MapCells[2].addGroupCell(0); //
		MapCells[2].addGroupCell(3);
		MapCells[2].addGroupCell(6);
		MapCells[3].addGroupCell(1); //
		MapCells[3].addGroupCell(2);
		MapCells[3].addGroupCell(5);
		MapCells[5].addGroupCell(0); //
		MapCells[5].addGroupCell(1);
		MapCells[5].addGroupCell(3);
		MapCells[6].addGroupCell(0); //
		MapCells[6].addGroupCell(1);
		MapCells[6].addGroupCell(2);
	}
	
	//get functions
	public Cell[] getMapCells()
	{
		return(MapCells);
	}

	public int getCellCount()
	{
		return(CellCount);
	}
	
	public int getTotalMobileHostsCount()
	{
		int tCount=0;
		for(int x=0;x<CellCount;x++)
		{
		  tCount+=MapCells[x].getMobileHostCount();
		}
		return(tCount);
	}
	
	public int getTotalMyChannelsCount()
	{
		int tCount=0;
		for(int x=0;x<CellCount;x++)
		{
		  tCount+=MapCells[x].getMyChannelCount();
		}
		return(tCount);
	}
	
	//returns which cell a mobileHost is present
	public int getCellIDOfMobileHost(int tMobileHostID)
	{
		int tCellID=-1;
		for(int t=0;t<CellCount;t++)
		{
			for(int j=0;j<MapCells[t].getMobileHostCount();j++)
			{
				if(MapCells[t].getMobileHostID(j)==tMobileHostID)
				{
					tCellID=t;
				}
			}
		}
		return(tCellID);
	}
	
	//returns index of a mobileHost inside a cell
	public int getMobileHostIndexInCell(int tMobileHostID)
	{
		int tIndex=-1;
		for(int t=0;t<CellCount;t++)
		{
			for(int j=0;j<MapCells[t].getMobileHostCount();j++)
			{
				if(MapCells[t].getMobileHostID(j)==tMobileHostID)
				{
					tIndex=j;
				}
			}
		}
		return(tIndex);
	}

	//returns which cell a mychannel is present
	public int getCellIDOfMyChannel(int tMyChannelID)
	{
		int tCellID=-1;
		for(int t=0;t<CellCount;t++)
		{
			for(int j=0;j<MapCells[t].getMyChannelCount();j++)
			{
				if(MapCells[t].getMyChannelID(j)==tMyChannelID)
				{
					tCellID=t;
				}
			}
		}
		return(tCellID);
	}
	
	//returns index of a mychannel inside a cell
	public int getMyChannelIndexInCell(int tMyChannelID)
	{
		int tIndex=-1;
		for(int t=0;t<CellCount;t++)
		{
			for(int j=0;j<MapCells[t].getMyChannelCount();j++)
			{
				if(MapCells[t].getMyChannelID(j)==tMyChannelID)
				{
					tIndex=j;
				}
			}
		}
		return(tIndex);
	}

	//methods
	public void draw(Graphics g)
	{
		try
		{
			//Thread.sleep(1);
			for(int t=0;t<CellCount;t++) MapCells[t].draw(g);
		}
		catch(Exception e)
		{
			System.out.println("Error: "+e.getMessage());
		}
	}
	
	//draw mobileHosts
	public void drawMobileHostsLabel(Graphics g)
	{
		g.setFont(new Font("Verdana",Font.BOLD,13));
		g.setColor(Color.BLUE);
		for(int i=0;i<CellCount;i++)
		{
			g.drawString("cell"+i,MapCells[i].getx()+20,MapCells[i].gety()+12);
		}
	}

	public String getAdjListString(int tCellIndex)
	{
		String tstr="";
		if(MapCells[tCellIndex].getAdjCount()!=0)
		{
			for(int ti=0;ti<MapCells[tCellIndex].getAdjCount();ti++)
			{
				tstr=tstr+Integer.toString(MapCells[tCellIndex].getAdjCell(ti))+" ";
			}
		}
		return(tstr);
	}

	public String getCellInfo()
	{
		//adjlist
		String tstr="";
		for(int t=0;t<CellCount;t++)
		{
			tstr+="cell"+MapCells[t].getid()+": [";
			tstr+=getAdjListString(t)+"]; ";
			tstr+=""+MapCells[t].getMobileHostCount()+" MHs; ";
			tstr+=""+MapCells[t].getMyChannelCount()+" Channels.";
			tstr+="\n";
		}
		return(tstr);
	}
};

class faultTolerantCN extends JFrame implements ActionListener,MouseMotionListener,MouseListener
{
	//ui
	JFrame frmMain=new JFrame("Fault-Tolerant Distributed Channel Allocation Scheme for Cellular Networks");
	JFrame frmResult=new JFrame("Result");
	JTextArea txtCellInfo=new JTextArea("");
	JTextArea txtChannelInfo=new JTextArea("");
	JTextArea txtDataStructures=new JTextArea("");
	JTextArea txtResult=new JTextArea("");
	JScrollPane spCellInfo=new JScrollPane(txtCellInfo);
	JScrollPane spChannelInfo=new JScrollPane(txtChannelInfo);
	JScrollPane spDataStructures=new JScrollPane(txtDataStructures);
	JScrollPane spResult=new JScrollPane(txtResult);
	JButton btStart=new JButton("Start");
	JLabel lblSimulationTime=new JLabel("");
	JLabel lblMobileHostCount=new JLabel("");
	JLabel lblXY=new JLabel("");
	JLabel lblCurrentMobileHost=new JLabel("");
	JLabel lblSourceMobileHost=new JLabel("");
	JLabel lblCurrentMyChannel=new JLabel("");
	JLabel lblAllocated=new JLabel("Allocated-Channel");
	JLabel lblBusy=new JLabel("Busy-Channel");
	JLabel lblRequestToMSS=new JLabel("Request-to-MSS");
	JLabel lblRequestToNeighbor=new JLabel("Request-to-Neighbor");
	Graphics g;
	
	//ui parameters
	int frmWidth=700,frmHeight=460;
	int mapWidth=420,mapHeight=340;
	
	//system parameters
	int currentMobileHost=-1;
	int sourceMobileHost=-1;
	int currentMyChannel=-1;
	boolean sourceSelected=false;
	int maxMobileHosts;
	int maxMyChannels;
	int radius=5;
	boolean ChannelAllocated=false;
	
	//channel allocation parameters
	boolean RequestMessageSent=false;
	
	Map map=new Map();
	Cell[] MapCells=map.getMapCells();
	javax.swing.Timer timerMain;
	long maptime=0;
	
	//constructor
	faultTolerantCN()
	{
		//get user-paramters
		maxMobileHosts=Integer.parseInt(JOptionPane.showInputDialog("Enter MobileHostCount: ","20"));
		maxMyChannels=Integer.parseInt(JOptionPane.showInputDialog("Enter ChannelCount: ","10"));
		
		frmMain.setDefaultLookAndFeelDecorated(true);
		frmMain.setResizable(false);
		frmMain.setBounds(20,60,frmWidth,frmHeight);
		frmMain.getContentPane().setLayout(null);
		frmMain.addMouseMotionListener(this);
		frmMain.addMouseListener(this);
		frmResult.setDefaultLookAndFeelDecorated(true);
		frmResult.setResizable(false);
		frmResult.setBounds(20+frmWidth+10,60,240,frmHeight);
		frmResult.getContentPane().setLayout(null);
		
		txtCellInfo.setEditable(false);
		spCellInfo.setBounds(frmWidth-200-30,30,200,100);
		frmMain.getContentPane().add(spCellInfo);
		txtChannelInfo.setEditable(false);
		spChannelInfo.setBounds(frmWidth-200-30,140,200,100);
		frmMain.getContentPane().add(spChannelInfo);
		txtDataStructures.setEditable(false);
		spDataStructures.setBounds(frmWidth-200-30,250,200,100);
		frmMain.getContentPane().add(spDataStructures);
		txtResult.setEditable(false);
		spResult.setBounds(20,20,200,frmHeight-160);
		frmResult.getContentPane().add(spResult);
		txtResult.setText("Result:\n");
		
		btStart.setBounds(frmWidth-200-30,360,200,25);
		frmMain.getContentPane().add(btStart);
		btStart.addActionListener(this);
		
		lblSimulationTime.setBounds(30,frmHeight-80,150,20);
		frmMain.getContentPane().add(lblSimulationTime);
		lblMobileHostCount.setBounds(30,frmHeight-60,150,20);
		frmMain.getContentPane().add(lblMobileHostCount);
		lblXY.setBounds(30,10,100,20);
		frmMain.getContentPane().add(lblXY);
		lblSourceMobileHost.setBounds(frmWidth-430,10,180,20);
		frmMain.getContentPane().add(lblSourceMobileHost);
		lblCurrentMobileHost.setBounds(frmWidth-430,frmHeight-80,180,20);
		frmMain.getContentPane().add(lblCurrentMobileHost);
		lblCurrentMyChannel.setBounds(frmWidth-430,frmHeight-60,180,20);
		frmMain.getContentPane().add(lblCurrentMyChannel);
		
		//set indicator labels
		lblAllocated.setBounds(70,frmHeight-130,150,20);
		frmResult.getContentPane().add(lblAllocated);
		lblBusy.setBounds(70,frmHeight-110,150,20);
		frmResult.getContentPane().add(lblBusy);
		lblRequestToMSS.setBounds(70,frmHeight-90,150,20);
		frmResult.getContentPane().add(lblRequestToMSS);
		lblRequestToNeighbor.setBounds(70,frmHeight-70,150,20);
		frmResult.getContentPane().add(lblRequestToNeighbor);

		frmMain.setVisible(true);
		frmResult.setVisible(true);
		try
		{
			//Thread.sleep(1);
		}
		catch(Exception e) { }

		g=frmMain.getGraphics();
		Graphics2D g2d=(Graphics2D)g;
		g2d.setStroke(new BasicStroke(2.0f));
		g2d.drawRect(30,60,mapWidth,mapHeight);
		Image i=new ImageIcon("images\\marea.gif").getImage();
		//Image i=new ImageIcon("images\\cbe_district_map.gif").getImage();
		//Image i=new ImageIcon("images\\cbe_city_map.jpg").getImage();
		g.drawImage(i,30,60,mapWidth,mapHeight,this);
		map.draw(g);
		map.drawMobileHostsLabel(g);

		//set indicators
		Graphics g1=frmResult.getGraphics();
		g1.setColor(new Color(0,0,0)); //allocated
		g1.drawRect(20,frmHeight-120+30,40,2);
		g1.setColor(new Color(255,0,255)); //busy
		g1.drawRect(20,frmHeight-100+30,40,2);
		g1.setColor(new Color(0,255,0)); //request-to-mss
		g1.drawRect(20,frmHeight-80+30,40,2);
		g1.setColor(new Color(0,0,255)); //request-to-neighbor
		g1.drawRect(20,frmHeight-60+30,40,2);

		//set timer
		timerMain=new javax.swing.Timer(1000,timerMain_Tick);
		timerMain.start();
		while(true);
	}

	public Map getMap()
	{
		return(map);
	}

	//events
	ActionListener timerMain_Tick=new ActionListener()
	{
		public void actionPerformed(ActionEvent e)
		{
			//add mobilehosts
			if(map.getTotalMobileHostsCount()>=maxMobileHosts)
			{
				timerMain.stop();
				addRandomMyChannels(maxMyChannels);
				displayStatus();
				updateChannelInfo();
				updateDataStructures();
				drawChannelLinks(4);
			}
			else
			{
				maptime++;
				lblSimulationTime.setText("Simulation Time: "+Long.toString(maptime));
				addRandomMobileHosts(5);
				lblMobileHostCount.setText("Total MobileHosts: "+map.getTotalMobileHostsCount());
			}
		}
	};
	
	public void actionPerformed(ActionEvent evt)
	{
		if(evt.getSource()==btStart)
		{
			if(sourceMobileHost!=-1)
			{
				SendRequestMessage();
				
				int tLocalCellID=map.getCellIDOfMobileHost(sourceMobileHost);
				CheckChannelsOfCell(tLocalCellID);
				if(ChannelAllocated==false)
				{
					addResultText("\n\nSending Request To Neighbors of "+tLocalCellID+": ["+map.getAdjListString(tLocalCellID)+"]");
					if(MapCells[tLocalCellID].getAdjCount()>0)
					{
						for(int t=0;t<MapCells[tLocalCellID].getAdjCount();t++)
						{
							DrawCellRequestArrow(tLocalCellID,MapCells[tLocalCellID].getAdjCell(t));
							CheckChannelsOfCell(MapCells[tLocalCellID].getAdjCell(t));
							if(ChannelAllocated==true) break;
						}
					}
				}
			}
		}
	}
	
	void SendRequestMessage()
	{
		if(RequestMessageSent==false)
		{
			addResultText("\nMH"+sourceMobileHost+" Sending Request Message to MSS"+map.getCellIDOfMobileHost(sourceMobileHost));
			int x1=MapCells[map.getCellIDOfMobileHost(sourceMobileHost)].getMobileHostX(map.getMobileHostIndexInCell(sourceMobileHost));
			int y1=MapCells[map.getCellIDOfMobileHost(sourceMobileHost)].getMobileHostY(map.getMobileHostIndexInCell(sourceMobileHost));
			int x2=MapCells[map.getCellIDOfMobileHost(sourceMobileHost)].getx();
			int y2=MapCells[map.getCellIDOfMobileHost(sourceMobileHost)].gety();
			g.setColor(Color.GREEN);
			int tCenterX=Globals.HexWidth/2;
			int tCenterY=Globals.HexHeight/2;
			g.drawLine(x1,y1,x2+tCenterX,y2+tCenterY);
			RequestMessageSent=true;
		}
	}
	
	void CheckChannelsOfCell(int tCellID)
	{
		addResultText("\n\nChecking Channels ["+getChannelsOfCell_String(tCellID)+"] of Cell"+tCellID);
		int tCount=MapCells[tCellID].getMyChannelCount();
		int selectedChannel=-1;
		if(tCount!=0)
		{
			for(int t=0;t<tCount;t++)
			{
				int tChannelID=MapCells[tCellID].getMyChannelID(t);
				//DrawChannelRequestArrow(tCellID,tChannelID);
				if(MapCells[tCellID].getMyChannelStatus(t)==ChannelStatus.Allocated)
				{
					if(selectedChannel==-1) selectedChannel=tChannelID;
				}
			}
		}
		if(selectedChannel==-1)
		{
			addResultText("\nNo Allocated Channels in Cell"+tCellID);
		}
		else
		{
			ChannelAllocated=true;
			ChangeChannelStatus(selectedChannel,ChannelStatus.Busy);
			addResultText("\nChannel"+selectedChannel+" allocated");
			updateChannelInfo();
			updateDataStructures();
			HighlightAllocatedChannel(selectedChannel);
		}
	}
	
	void ChangeChannelStatus(int tChannelID,int tNewStatus)
	{
		MapCells[map.getCellIDOfMyChannel(tChannelID)].setMyChannelStatus(map.getMyChannelIndexInCell(tChannelID),tNewStatus);
		updateChannelInfo();
	}
	
	void DrawChannelRequestArrow(int tCellID,int tChannelID)
	{
		int tCenterX=Globals.HexWidth/2;
		int tCenterY=Globals.HexHeight/2;
		int x1=MapCells[tCellID].getx()+tCenterX;
		int y1=MapCells[tCellID].gety()+tCenterY;
		int x2=MapCells[map.getCellIDOfMyChannel(tChannelID)].getMyChannelX(map.getMyChannelIndexInCell(tChannelID));
		int y2=MapCells[map.getCellIDOfMyChannel(tChannelID)].getMyChannelY(map.getMyChannelIndexInCell(tChannelID));
		g.setColor(new Color(128,0,0));
		g.drawLine(x1,y1,x2,y2);
	}
	
	void DrawCellRequestArrow(int tCellID1,int tCellID2)
	{
		int tCenterX=Globals.HexWidth/2;
		int tCenterY=Globals.HexHeight/2;
		int x1=MapCells[tCellID1].getx()+tCenterX;
		int y1=MapCells[tCellID1].gety()+tCenterY;
		int x2=MapCells[tCellID2].getx()+tCenterX;
		int y2=MapCells[tCellID2].gety()+tCenterY;
		g.setColor(new Color(0,0,255));
		g.drawLine(x1,y1,x2,y2);
	}
	
	void HighlightAllocatedChannel(int tChannelID)
	{
		int tx=MapCells[map.getCellIDOfMyChannel(tChannelID)].getMyChannelX(map.getMyChannelIndexInCell(tChannelID));
		int ty=MapCells[map.getCellIDOfMyChannel(tChannelID)].getMyChannelY(map.getMyChannelIndexInCell(tChannelID));
		g.setColor(new Color(255,0,255));
		g.drawOval(tx-4,ty-4,10,10);
	}
	
	ArrayList getChannelsOfCell(int tCellID)
	{
		ArrayList alChannels=new ArrayList();
		
		for(int t=0;t<MapCells[tCellID].getMyChannelCount();t++)
		{
			int tChannelID=MapCells[tCellID].getMyChannelID(t);
			Integer int1=new Integer(tChannelID);
			alChannels.add(int1);
		}
		
		return(alChannels);
	}
	
	String getChannelsOfCell_String(int tCellID)
	{
		String tstr="";
		ArrayList al1=getChannelsOfCell(tCellID);
		for(int t=0;t<al1.size();t++) tstr+=((Integer)al1.get(t)).intValue()+",";
		if(tstr.length()!=0) tstr=tstr.substring(0,tstr.length()-1);
		return(tstr);
	}
	
	ArrayList getChannelsInStatus(int tStatus)
	{
		ArrayList alStatus=new ArrayList();
		
		for(int t=0;t<map.getTotalMyChannelsCount();t++)
		{
			int tStatus1=MapCells[map.getCellIDOfMyChannel(t)].getMyChannelStatus(map.getMyChannelIndexInCell(t));
			if(tStatus==tStatus1)
			{
				Integer int1=new Integer(t);
				alStatus.add(int1);
			}
		}
		
		return(alStatus);
	}
	
	String getChannelsInStatus_String(int tStatus)
	{
		String tstr="";
		ArrayList al1=getChannelsInStatus(tStatus);
		for(int t=0;t<al1.size();t++) tstr+=((Integer)al1.get(t)).intValue()+",";
		if(tstr.length()!=0) tstr=tstr.substring(0,tstr.length()-1);
		return(tstr);
	}

	ArrayList getChannelsOfCellInStatus(int tCellID,int tStatus)
	{
		ArrayList alChannels=new ArrayList();
		
		for(int t=0;t<MapCells[tCellID].getMyChannelCount();t++)
		{
			int tStatus1=MapCells[tCellID].getMyChannelStatus(t);
			if(tStatus1==tStatus)
			{
				int tChannelID=MapCells[tCellID].getMyChannelID(t);
				Integer int1=new Integer(tChannelID);
				alChannels.add(int1);
			}
		}
		
		return(alChannels);
	}
	
	String getChannelsOfCellInStatus_String(int tCellID,int tStatus)
	{
		String tstr="";
		ArrayList al1=getChannelsOfCellInStatus(tCellID,tStatus);
		for(int t=0;t<al1.size();t++) tstr+=((Integer)al1.get(t)).intValue()+",";
		if(tstr.length()!=0) tstr=tstr.substring(0,tstr.length()-1);
		return(tstr);
	}
	
	void updateChannelInfo()
	{
		String tstr="Channel Info:\n\n";
		
		tstr+="Allocated: "+getChannelsInStatus_String(ChannelStatus.Allocated)+"\n";
		tstr+="Busy: "+getChannelsInStatus_String(ChannelStatus.Busy)+"\n";
		
		txtChannelInfo.setText(tstr);
		txtCellInfo.setText("Cell-Info:\n\n"+map.getCellInfo());
	}
	
	void updateDataStructures()
	{
		String tstr="";
		
		for(int t=0;t<map.getCellCount();t++)
		{
			tstr+="Cell"+t+":\n";
			tstr+="Allocated("+t+"): "+getChannelsOfCellInStatus_String(t,ChannelStatus.Allocated)+"\n";
			tstr+="Busy("+t+"): "+getChannelsOfCellInStatus_String(t,ChannelStatus.Busy)+"\n\n";
		}
		
		txtDataStructures.setText("Data Structures:\n\n"+tstr);
	}
	
	void drawChannelLinks(int tCellID)
	{
		for(int j=0;j<MapCells[tCellID].getMyChannelCount();j++)
		{
			int tStatus1=MapCells[tCellID].getMyChannelStatus(j);
			if(tStatus1==ChannelStatus.Busy)
			{
				g.setColor(new Color(255,0,255));
			}
			else if(tStatus1==ChannelStatus.Allocated)
			{
				g.setColor(new Color(0,0,0));
			}
			
			int tMobileHost=MapCells[tCellID].getMyChannelLink(j);
			int x1=MapCells[tCellID].getMobileHostX(map.getMobileHostIndexInCell(tMobileHost));
			int y1=MapCells[tCellID].getMobileHostY(map.getMobileHostIndexInCell(tMobileHost));
			int x2=MapCells[tCellID].getMyChannelX(j);
			int y2=MapCells[tCellID].getMyChannelY(j);
			g.drawLine(x1,y1,x2,y2);
		}
	}
	
	void swapChannelStatus(int tChannelID)
	{
		int tCellID=map.getCellIDOfMyChannel(tChannelID);
		int tStatus=MapCells[tCellID].getMyChannelStatus(map.getMyChannelIndexInCell(tChannelID));
		ChangeChannelStatus(tChannelID,tStatus==ChannelStatus.Allocated?ChannelStatus.Busy:ChannelStatus.Allocated);
		updateChannelInfo();
		updateDataStructures();
		drawChannelLinks(tCellID);
		displayStatus();
	}
	
	//algorithms
	void drawPath(int tindex1,int tindex2)
	{
		int x1=MapCells[map.getCellIDOfMobileHost(tindex1)].getMobileHostX(map.getMobileHostIndexInCell(tindex1));
		int y1=MapCells[map.getCellIDOfMobileHost(tindex1)].getMobileHostY(map.getMobileHostIndexInCell(tindex1));
		int x2=MapCells[map.getCellIDOfMobileHost(tindex2)].getMobileHostX(map.getMobileHostIndexInCell(tindex2));
		int y2=MapCells[map.getCellIDOfMobileHost(tindex2)].getMobileHostY(map.getMobileHostIndexInCell(tindex2));
		g.setColor(Color.GREEN);
		g.drawLine(x1,y1,x2,y2);
	}

	//find euclidean distance between geographical mobilehosts
	int getDistance(int x1,int y1,int x2,int y2)
	{
		double d=Math.sqrt(Math.pow((double)(x2-x1),2.0)+Math.pow((double)(y2-y1),2.0));
		return((int)d);
	}

	//utility functions
	boolean between(int x,int y,int x1,int y1,int x2,int y2)
	{
		boolean flag=false;
		
		if(x>=x1&&x<=x2)
		{
			if(y>=y1&&y<=y2)
			{
				flag=true;
			}
		}
		
		return(flag);
	}
	
	public void displayStatus()
	{
		lblSourceMobileHost.setText("Source MobileHost: "+sourceMobileHost);
		lblCurrentMobileHost.setText("Current MobileHost: "+currentMobileHost);
		lblCurrentMyChannel.setText("Current Channel: "+currentMyChannel);
		//draw source mobilehost in different color
		if(sourceMobileHost!=-1)
		{
			int tx=MapCells[map.getCellIDOfMobileHost(sourceMobileHost)].getMobileHostX(map.getMobileHostIndexInCell(sourceMobileHost));
			int ty=MapCells[map.getCellIDOfMobileHost(sourceMobileHost)].getMobileHostY(map.getMobileHostIndexInCell(sourceMobileHost));
			g.setColor(new Color(0,255,0));
			g.drawOval(tx,ty,radius,radius);
		}
		//display channel status
		if(currentMyChannel!=-1)
		{
			int tStatus=MapCells[map.getCellIDOfMyChannel(currentMyChannel)].getMyChannelStatus(map.getMyChannelIndexInCell(currentMyChannel));
			lblCurrentMyChannel.setText("Current Channel: "+currentMyChannel+" ("+ChannelStatus.StatusString[tStatus]+")");
		}
	}

	//mouse motion listener methods
	public void mouseDragged(MouseEvent me)
	{
		//
	}
	
	public void mouseMoved(MouseEvent me)
	{
		int xposition=me.getX();
		int yposition=me.getY();
		lblXY.setText("("+xposition+","+yposition+")");
		
		int n=map.getTotalMobileHostsCount();
		int trad=radius-2;

		//check currentmobilehost
		for(int i=0;i<map.getCellCount();i++)
		{
			for(int j=0;j<MapCells[i].getMobileHostCount();j++)
			{
				int x1=MapCells[i].getMobileHostX(j)-trad;
				int x2=MapCells[i].getMobileHostX(j)+trad;
				int y1=MapCells[i].getMobileHostY(j)-trad;
				int y2=MapCells[i].getMobileHostY(j)+trad;

				if(between(xposition,yposition,x1,y1,x2,y2)==true)
				{
					currentMobileHost=MapCells[i].getMobileHostID(j);
					displayStatus();
				}
			}
		}

		//check currentmychannel
		for(int i=0;i<map.getCellCount();i++)
		{
			for(int j=0;j<MapCells[i].getMyChannelCount();j++)
			{
				int x1=MapCells[i].getMyChannelX(j)-trad;
				int x2=MapCells[i].getMyChannelX(j)+trad;
				int y1=MapCells[i].getMyChannelY(j)-trad;
				int y2=MapCells[i].getMyChannelY(j)+trad;

				if(between(xposition,yposition,x1,y1,x2,y2)==true)
				{
					currentMyChannel=MapCells[i].getMyChannelID(j);
					displayStatus();
				}
			}
		}
	}
	
	//mouselistener methods
	public void mouseClicked(MouseEvent me)
	{
		if(currentMobileHost>=0)
		{
			if(sourceSelected==false)
			{
				sourceMobileHost=currentMobileHost;
				sourceSelected=true;
			}
			displayStatus();
		}
		
		//change channel status if right-clicked
		int mask=InputEvent.BUTTON1_MASK-1;
		int mods=me.getModifiers()&mask;
		if(mods==4) //right-clicked
		{
			if(currentMyChannel>=0)
			{
				swapChannelStatus(currentMyChannel);
			}
		}
	}
	
	public void mouseEntered(MouseEvent me)
	{
		//
	}
	
	public void mouseExited(MouseEvent me)
	{
		//
	}
	
	public void mousePressed(MouseEvent me)
	{
		//
	}
	
	public void mouseReleased(MouseEvent me)
	{
		//
	}
	
	//methods
	void addResultText(String tStr)
	{
		txtResult.setText(txtResult.getText()+tStr+".");
	}
	
	void addRandomMobileHosts(int tCount)
	{
		for(int i=0;i<tCount;i++)
		{
			try
			{
				double random=java.lang.Math.random();
				int no=(int)(map.getCellCount()*random);
				if(map.getTotalMobileHostsCount()==0) no=4;
				double random1=java.lang.Math.random();
				int no1=(int)(((double)Globals.HexWidth-5.0)*random1);
				double random2=java.lang.Math.random();
				int no2=(int)(((double)Globals.HexHeight-5.0)*random2);
				Graphics2D g2d=(Graphics2D)g;
				g2d.setStroke(new BasicStroke(2.0f));
				g.setColor(new Color(255,0,0));
				int xposition=MapCells[no].getx()+no1;
				int yposition=MapCells[no].gety()+no2;
				g.drawOval(xposition,yposition,radius,radius);
				MapCells[no].addMobileHost(xposition,yposition);
			}
			catch(Exception ex)
			{
				System.out.println("Error: "+ex.getMessage());
			}
		}
	}
	
	void addRandomMyChannels(int tCount)
	{
		for(int i=0;i<tCount;i++)
		{
			try
			{
				double random=java.lang.Math.random();
				int tCellID=(int)(map.getCellCount()*random);
				if(map.getTotalMyChannelsCount()==0) tCellID=4;
				addNewChannelToCell(tCellID);
			}
			catch(Exception ex)
			{
				System.out.println("Error: "+ex.getMessage());
			}
		}
	}
	
	void addNewChannelToCell(int tCellID)
	{
		double random1=java.lang.Math.random(); //random x
		int no1=(int)(((double)Globals.HexWidth-5.0)*random1);
		double random2=java.lang.Math.random(); //random y
		int no2=(int)(((double)Globals.HexHeight-5.0)*random2);
		double random3=java.lang.Math.random(); //random status
		int no3=(int)(2*random3);
		int tx=MapCells[tCellID].getx()+no1;
		int ty=MapCells[tCellID].gety()+no2;
		if(MapCells[tCellID].addMyChannel(tx,ty,no3)==true)
		{
			g.setColor(new Color(128,0,0));
			g.drawRect(tx,ty,3,3);
		}
	}

	public static void main(String args[])
	{
		new faultTolerantCN();
	}
}
